#!/bin/bash

####
# borg wrapper script
# expects configuration in /etc/borg/<PROFILE>/conf
####

function usage() {
	echo "Usage: "
	echo "$0 backup -p <profile_name> -c <backup|list|init|prune|info> -o <additional options - parsed to borg>"
	echo "Info about last backup: $0 backup -p <profile_name> -c info" 
	echo "List all files in backup: $0 backup -p <profile_name> -c list-files -b <BACKUP-NAME>" 
	echo "Extract (restore): $0 backup -p <profile_name> -c extract -b <BACKUP-NAME> -x "FILE/PATTERN"" 
}

while getopts ":p:c:o:b:x:" opt; do
	case $opt in
		p)
			borg_profile=$OPTARG
		;;
		c)
			command_type=$OPTARG
		;;
		b)
			backup_name=$OPTARG
		;;
		o)
			borg_add_options=$OPTARG
		;;

		x)
			extract_pattern=$OPTARG
		;;

		\?)
			echo "Invalid option: -$OPTARG" >&2
			usage
			exit 1
		;;
		:)
			echo "Option -$OPTARG requires an argument." >&2
			exit 1
		;;
	esac
done

if [ "$borg_profile" == "" ]; then
	echo "You have to specify the profile"
	usage
		exit 1
else
	if test -f /etc/borg/$borg_profile/conf; then
		source /etc/borg/$borg_profile/conf
	else
		echo "There is no profile like \"$borg_profile\". Please check config"
		exit 1
	fi
fi

# create logdir
if ! test -d $(dirname $borg_logfile); then
	echo "create logdir"
	mkdir -p $(dirname $borg_logfile)
fi

# generate excludes 
exclude_dirs=""
for i in $borg_exclude; do
    exclude_dirs="${exclude_dirs}--exclude $i ";
done

if test -f /etc/borg/$borg_profile/exclude; then
    borg_exclude_from="--exclude-from /etc/borg/$borg_profile/exclude"
fi

if [ "$borg_localrepo" == "1" ]; then
    borg_repopath=$borg_path/$borg_repo
else
    borg_repopath=$borg_path:$borg_repo
fi

function borg_backup() {
	SECONDS=0
	echo "$borg_repo: Start der Sicherung $(date)." | tee -a $borg_logfile
	echo "Quelle: $backup_source"
	echo "Exclude: $exclude_dirs"
	echo "Exclude-File: $borg_exclude_from"
	echo "Exclude-Tag: $borg_exclude_tag"
	echo "with additional options: " $borg_options

    if [ "$all_filesystems" == "1" ]; then
        one_file_system=""
    else
        one_file_system="--one-file-system"
    fi
    
    $borg_command create --compression $borg_compression $borg_options \
        $one_file_system --exclude-caches $exclude_dirs $borg_exclude_from \
        $borg_exclude_tag $borg_add_options \
        $borg_repopath::'{hostname}-{now:%Y-%m-%dT%H:%M:%S}' $backup_source | tee -a $borg_logfile

	echo "$borg_repo: Ende der Sicherung $(date). Dauer: $SECONDS Sekunden" | tee -a $borg_logfile
}

function borg_prune() {
	echo "Pruning Repo whith $borg_prune"
	$borg_command prune -v $borg_repopath --prefix '{hostname}-' $borg_prune $borg_add_options
}

function borg_pre() {
	test /etc/borg/$borg_profile/pre && source /etc/borg/$borg_profile/pre
}

function borg_post() {
	test -f /etc/borg/$borg_profile/post && source /etc/borg/$borg_profile/post
}


case $command_type in
	backup)
		borg_pre
		borg_backup
		borg_post
		borg_prune
	;;

	pre)
		borg_pre
	;;

	post)
		borg_post
	;;

	list)
		$borg_command list $borg_repopath
	;;

	list-files)
		if [ "$backup_name" == "" ]; then
			echo "please specify backup name - look $0 $borg_profile -c list"
		else
			$borg_command list $borg_repopath::$backup_name
		fi
	;;

        info)   
                # info about last backup
                last_backup=$($borg_command list $borg_repopath | tail -1 |cut -d' ' -f1)
		# hier sollte noch getestet werden, ob das lezte backup nur ein checkpoint war ...
		if echo $last_backup | grep checkpoint; then
			echo "The last Backup was only a snapshot. Please check!"
		else
			$borg_command info $borg_repopath::$last_backup
		fi
        ;;

	extract)
		if [ "$backup_name" == "" ]; then
			echo "Please specify backup name and pattern to extract."
			echo "(Pattern without leading \"/\".)"
		else
            $borg_command extract $borg_add_options $borg_repopath::$backup_name $extract_pattern
		fi
	;;

	init)
		$borg_command init -e $borg_encryption $borg_repopath
	;;

	prune)
		borg_prune
	;;

	*)
		usage
	;;
esac

unset BORG_PASSPHRASE
exit 0
